#include "gpdef.h"
#include "gpstdio.h"
#include "gpwrap.h"
#include "gpstdlib.h"
#include "gpmem.h"
#include "types.h"

#include "chicken2.h"
#include "gfx.h"
#include "xlat.h"

GPDRAWSURFACE gpDraw[2];
short ACTIVE_SURFACE;

struct {
	unsigned char xsize, ysize, xhotspot, yhotspot;
	unsigned short size, offset;
} pobdata[38];

// Allocates memory using malloc and returns a pointer to the memory
// or zero on failure.
ubyte *allocate_mem(size_t amount) {
  ubyte *pointer;
  if ((pointer = (ubyte *)gm_malloc(amount)) == NULL) {
    return 0;
  }
  return pointer;
}

// Returns a random integer between 0 and range.
long random_nr(long range) {
  return (GpRandN(range));
}

// Gives the time as a random seed.
void randomize(void) {
  GpSrand((int)GpTickCountGet());
}

void init_gfx(void) {
  int i;
  for(i = 0 ; i < 2 ; i++) GpLcdSurfaceGet(&gpDraw[i], i); 
  GpGraphicModeSet(8, NULL);
  GpSurfaceSet(&gpDraw[0]);
  GpLcdEnable();
  ACTIVE_SURFACE = 1;
}

// Fills 76800 bytes at mem with value.
void clear_screen(ubyte *mem, unsigned long value) {
	gm_memset(mem, value, 76800);
}

// Copies 76800 bytes from source to dest.
void copy_screen(ubyte *dest, ubyte *source) {
    gm_memcpy(dest, source, 76800);
}

// Sets a palette.
void set_palette(ubyte *pal) {
  int i;
  ubyte r, g, b;
  for( i = 0 ; i < 256 ; i++ ){
	r = pal[i * 3    ]>>1;
	g = pal[i * 3 + 1]>>1;
	b = pal[i * 3 + 2]>>1;
	HARDWARE_PALETTE[i] = ((r&0x1f)<<11) | ((g&0x1f)<<6) | ((b&0x1f)<<1) | 0;
  }
}

void palette_backup(unsigned int *PaletteBackup)
{
	int color;
	for (color=0;color<256;color++) 
		PaletteBackup[color]=HARDWARE_PALETTE[color];	
}

void palette_restore(unsigned int *PaletteBackup)
{
	int color;
	for (color=0;color<256;color++) 
		HARDWARE_PALETTE[color]=PaletteBackup[color];	
	
}

void fade_out(int speed)
{
	unsigned char r,v,b;
	unsigned int temp;
	short i ;

	do {
		temp=0;
		for (i=0;i<255;i++)
		{
			r= (HARDWARE_PALETTE[i]>>11) & 0x1f;
			v= (HARDWARE_PALETTE[i]>> 6) & 0x1f;
			b= (HARDWARE_PALETTE[i]>> 1) & 0x1f;
			if (r) r--;
			if (v) v--;
			if (b) b--;
			if (r+v+b) temp++;
			HARDWARE_PALETTE[i] = (r<<11) | (v<<6) | (b<<1) | 0 ;
		}
		
		sleep(speed);
		if (!temp) break;
		
	} while(1) ;
}


// Puts a pixel in mem.
// Clipping.
void put_pixel(short x, short y, unsigned char color, ubyte *mem) {
  if ((x > -1) && (x < 320) && (y > -1) && (y < 240)) {
    mem[(y << 8) + (y << 6) + x] = color;
  }
}

/*void fade(char *pal1, char *pal2, short speed) {
  short c, c2;
  char curpal[768];
  memcpy(curpal, pal1, 768);
  for (c2 = 0; c2 < speed; c2++) {
    for (c = 0; c < 768; c++) {
      curpal[c] = pal1[c] + ((float)(pal2[c] - pal1[c]) / speed) * c2;
    }
    set_palette(curpal);
  }
}*/


void load_font(char *filename, char nr) {
  FILE *infile;
  if ((infile = gpfopen(filename, "rb")) == NULL) return;
  font[nr].width = fgetc(infile);
  font[nr].height = fgetc(infile);
  font[nr].offset = (char *)gm_malloc(256 * font[nr].height * font[nr].width);
  fread(font[nr].offset, 1, 256 * font[nr].height * font[nr].width, infile);
  gpfclose(infile);
}

void put_pob(short xpos, short ypos, short n, ubyte *data, ubyte *dest) {
  unsigned short offs, poboffs, x, y, w, c;
  unsigned char color;
  short xleft, xwidth, yupper, yheight;

  data += pobdata[n].offset;

  // Partially inside screen?
  if (((xpos - pobdata[n].xhotspot + pobdata[n].xsize) > -1) && ((xpos - pobdata[n].xhotspot) < 320) &&
     ((ypos - pobdata[n].yhotspot + pobdata[n].ysize) > -1) && ((ypos - pobdata[n].yhotspot) < 240)) {
    // Entierly inside screen?
    if (((xpos - pobdata[n].xhotspot) > -1) && ((xpos - pobdata[n].xhotspot + pobdata[n].xsize) < 320) &&
       ((ypos - pobdata[n].yhotspot) > -1) && ((ypos - pobdata[n].yhotspot + pobdata[n].ysize) < 240)) {

      poboffs = 0;
      offs = (ypos - pobdata[n].yhotspot) * 320 + xpos - pobdata[n].xhotspot;
      for (y = 0; y < pobdata[n].ysize; y++) {
        for (x = 0; x < pobdata[n].xsize; x++) {
          color = data[poboffs++];
          if (color != 0) dest[offs] = color;
          offs++;
        }
        offs += 320 - pobdata[n].xsize;
      }
    }

    // Clipping is needed
    else {

      xleft = pobdata[n].xhotspot - xpos;
      if (xleft < 0) xleft = 0;
      xwidth = 320 - xpos + pobdata[n].xhotspot;
      if (xwidth > pobdata[n].xsize) xwidth = pobdata[n].xsize;

      yupper = pobdata[n].yhotspot - ypos;
      if (yupper < 0) yupper = 0;
      yheight = 240 - ypos + pobdata[n].yhotspot;
      if (yheight > pobdata[n].ysize) yheight = pobdata[n].ysize;

      poboffs = yupper * pobdata[n].xsize + xleft;
      offs = (ypos - pobdata[n].yhotspot + yupper) * 320 + xpos - pobdata[n].xhotspot + xleft;

      for (y = yupper; y < yheight; y++) {
        for (x = xleft; x < xwidth; x++) {
          color = data[poboffs++];
          if (color != 0) dest[offs] = color;
          offs++;
        }
        poboffs += xleft + (pobdata[n].xsize - xwidth);
        offs += 320 - (xwidth - xleft);
      }
    }
  }
}

void load_pobs(char *filename, ubyte *dest) {
	FILE *infile;
	unsigned short offs;
	short x, y, c, c2, size;

	infile = gpfopen(filename, "rb");
	offs = 0;

	for (c2 = 0; c2 < 38; c2++) {
		x = fgetc(infile);
		y = fgetc(infile);
		size = y * x;
		pobdata[c2].xsize = x;
		pobdata[c2].ysize = y;
		pobdata[c2].size = size;
		x = fgetc(infile);
		y = fgetc(infile);
		pobdata[c2].xhotspot = x;
		pobdata[c2].yhotspot = y;
		pobdata[c2].offset = offs;
		for (c = 0; c < size; c++) {
			dest[offs++] = fgetc(infile);
		}
	}
	gpfclose(infile);
}
